Package: RGraphSpace
Type: Package
Version: 1.3.0
Title: A Lightweight Interface Between 'igraph' and 'ggplot2' Graphics
Description: An interface to integrate 'igraph' and 'ggplot2' graphics 
    within a normalized coordinate system. 'RGraphSpace' implements 
    geometric objects based on 'ggplot2' prototypes, optimized for 
    the representation of large networks. The package provides three 
    specialized 'geoms' to translate graph data into geometric layers, 
    supporting customization of aesthetics and visual styles. These 
    'geoms' use a dual-anchor normalization approach to align layers, 
    required for analyses where network elements must be referenced 
    to a spatial map. 'RGraphSpace' aims to facilitate side-by-side 
    visualization of multiple graphs spatially aligned with reference 
    maps and images.
Authors@R: 
    c(person("Sysbiolab Team", role = "aut"),
      person("Flávio", "Kessler", role = "ctb"),
      person("Jonathan", "Back", role = "ctb"),
      person("Lana", "Querne", role = "ctb"),
      person("Victor", "Apolonio", role = c("ctb")),
      person("Vinicius", "Chagas", role = "ctb"),
      person("Mauro", "Castro", role = c("cre"),
              email = "mauro.a.castro@gmail.com",
              comment = c(ORCID = "0000-0003-4942-8131")))
Depends: R(>= 4.5), methods, ggplot2
Imports: grDevices, grid, igraph, tidygraph, scales, rlang, ggrastr,
        lifecycle
Suggests: knitr, rmarkdown, testthat, ggraph, ggnewscale, patchwork
Enhances: RedeR
License: Artistic-2.0
VignetteBuilder: knitr
URL: https://github.com/sysbiolab/RGraphSpace,
        https://sysbiolab.github.io/RGraphSpace/
BugReports: https://github.com/sysbiolab/RGraphSpace/issues
Collate: 'geom-edgespace.R' 'geom-graphspace.R' 'geom-nodespace.R'
        'gspace-checks.R' 'gspace-classes.R' 'gspace-constructor.R'
        'gspace-generics.R' 'gspace-methods.R' 'gspace-misc.R'
        'gspace-normalize.R' 'gspace-supplements.R' 'gspace-themes.R'
        'gspace-validation.R' 'inject-nodespace.R'
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-21 15:30:26 UTC; maac
Author: Sysbiolab Team [aut],
  Flávio Kessler [ctb],
  Jonathan Back [ctb],
  Lana Querne [ctb],
  Victor Apolonio [ctb],
  Vinicius Chagas [ctb],
  Mauro Castro [cre] (ORCID: <https://orcid.org/0000-0003-4942-8131>)
Maintainer: Mauro Castro <mauro.a.castro@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-22 06:30:02 UTC
Built: R 4.6.0; ; 2026-05-25 04:25:31 UTC; unix
