Package: RGraphSpace
Type: Package
Version: 1.4.0
Title: A Lightweight Interface Between 'igraph' and 'ggplot2' Graphics
Description: An interface to integrate 'igraph' and 'ggplot2' graphics within 
    a normalized coordinate system. 'RGraphSpace' extends 'ggplot2' with 
    graph-aware geometries optimized for large networks. The 'GraphSpace' class 
    integrates directly with 'ggplot2' through specialized 'geoms' and lazy 
    resolution of node attributes, supporting customization of aesthetics and 
    visual styles. These 'geoms' use a dual-anchor normalization approach to 
    align layers, particularly useful for analyses in which network elements   
    must be spatially aligned with reference maps and images. 
Authors@R: 
    c(person("Sysbiolab Team", role = "aut"),
      person("Flávio", "Kessler", role = "ctb"),
      person("Jonathan", "Back", role = "ctb"),
      person("Lana", "Querne", role = "ctb"),
      person("Victor", "Apolonio", role = c("ctb")),
      person("Vinicius", "Chagas", role = "ctb"),
      person("Mauro", "Castro", role = c("cre"),
              email = "mauro.a.castro@gmail.com",
              comment = c(ORCID = "0000-0003-4942-8131")))
Depends: R(>= 4.5), methods, ggplot2 (>= 4.0)
Imports: grDevices, grid, igraph, tidygraph, scales, rlang, ggrastr,
        Matrix, lifecycle
Suggests: knitr, rmarkdown, testthat, ggraph, ggnewscale, patchwork,
        Seurat, SeuratObject
Enhances: RedeR
License: Artistic-2.0
VignetteBuilder: knitr
URL: https://github.com/sysbiolab/RGraphSpace,
        https://sysbiolab.github.io/RGraphSpace/
BugReports: https://github.com/sysbiolab/RGraphSpace/issues
Collate: 'geom-edgespace.R' 'geom-graphspace.R' 'geom-nodespace.R'
        'gspace-checks.R' 'gspace-classes.R' 'gspace-constructor.R'
        'gspace-generics.R' 'gspace-methods.R' 'gspace-misc.R'
        'gspace-normalize.R' 'gspace-supplements.R' 'gspace-themes.R'
        'gspace-validation.R' 'gspace-ggplot-constructor.R'
        'annotation-gspace.R' 'gspace-coercion.R' 'gspace-features.R'
Encoding: UTF-8
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-06-12 21:32:39 UTC; maac
Author: Sysbiolab Team [aut],
  Flávio Kessler [ctb],
  Jonathan Back [ctb],
  Lana Querne [ctb],
  Victor Apolonio [ctb],
  Vinicius Chagas [ctb],
  Mauro Castro [cre] (ORCID: <https://orcid.org/0000-0003-4942-8131>)
Maintainer: Mauro Castro <mauro.a.castro@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-13 05:10:23 UTC
