# Generated by roxygen2: do not edit by hand

S3method(.DollarNames,GraphSpace)
S3method(as.igraph,GraphSpace)
S3method(fortify,GraphSpace)
S3method(ggplot_add,inject_nodespace)
S3method(plot,GraphSpace)
export(GeomEdgeSpace)
export(GeomGraphSpace)
export(GeomNodeSpace)
export(StatEdgeSpace)
export(StatNodeSpace)
export(as_colorraster)
export(edgespace_handler)
export(geom_edgespace)
export(geom_graphspace)
export(geom_nodespace)
export(inject_nodespace)
export(nodespace_handler)
export(theme_gspace_coords)
export(theme_gspace_legend)
export(theme_gspace_th0)
export(theme_gspace_th1)
export(theme_gspace_th2)
export(theme_gspace_th3)
exportClasses(GraphSpace)
exportMethods("$")
exportMethods("$<-")
exportMethods("gs_edge_attr<-")
exportMethods("gs_vertex_attr<-")
exportMethods("names<-")
exportMethods(GraphSpace)
exportMethods(cropGraphSpace)
exportMethods(getGraphSpace)
exportMethods(gs_ecount)
exportMethods(gs_edge_attr)
exportMethods(gs_edges)
exportMethods(gs_graph)
exportMethods(gs_nodes)
exportMethods(gs_vcount)
exportMethods(gs_vertex_attr)
exportMethods(names)
exportMethods(normalizeGraphSpace)
exportMethods(plotGraphSpace)
import(igraph)
import(methods)
importFrom(ggplot2,"%+replace%")
importFrom(ggplot2,.pt)
importFrom(ggplot2,Geom)
importFrom(ggplot2,aes)
importFrom(ggplot2,annotation_raster)
importFrom(ggplot2,coord_fixed)
importFrom(ggplot2,draw_key_point)
importFrom(ggplot2,element_blank)
importFrom(ggplot2,element_line)
importFrom(ggplot2,element_rect)
importFrom(ggplot2,element_text)
importFrom(ggplot2,expansion)
importFrom(ggplot2,fortify)
importFrom(ggplot2,from_theme)
importFrom(ggplot2,geom_blank)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_segment)
importFrom(ggplot2,geom_text)
importFrom(ggplot2,gg_par)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,ggplot_add)
importFrom(ggplot2,ggproto)
importFrom(ggplot2,guide_legend)
importFrom(ggplot2,guides)
importFrom(ggplot2,is_waiver)
importFrom(ggplot2,labs)
importFrom(ggplot2,layer)
importFrom(ggplot2,margin)
importFrom(ggplot2,remove_missing)
importFrom(ggplot2,scale_colour_identity)
importFrom(ggplot2,scale_linetype_manual)
importFrom(ggplot2,scale_x_continuous)
importFrom(ggplot2,scale_y_continuous)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(ggplot2,theme_gray)
importFrom(ggplot2,translate_shape_string)
importFrom(ggplot2,unit)
importFrom(ggplot2,zeroGrob)
importFrom(ggrastr,rasterize)
importFrom(grDevices,as.raster)
importFrom(grDevices,col2rgb)
importFrom(grDevices,hcl.colors)
importFrom(grDevices,is.raster)
importFrom(graphics,plot)
importFrom(grid,arrow)
importFrom(grid,gList)
importFrom(grid,gpar)
importFrom(grid,grobName)
importFrom(grid,grobTree)
importFrom(grid,pointsGrob)
importFrom(grid,segmentsGrob)
importFrom(grid,unit)
importFrom(igraph,'E<-')
importFrom(igraph,'V<-')
importFrom(igraph,'edge_attr<-')
importFrom(igraph,E)
importFrom(igraph,V)
importFrom(igraph,any_loop)
importFrom(igraph,any_multiple)
importFrom(igraph,as.igraph)
importFrom(igraph,as_adjacency_matrix)
importFrom(igraph,as_edgelist)
importFrom(igraph,as_undirected)
importFrom(igraph,degree)
importFrom(igraph,delete_edge_attr)
importFrom(igraph,delete_graph_attr)
importFrom(igraph,delete_vertex_attr)
importFrom(igraph,ecount)
importFrom(igraph,edge_attr)
importFrom(igraph,edge_attr_names)
importFrom(igraph,empty_graph)
importFrom(igraph,ends)
importFrom(igraph,graph_attr_names)
importFrom(igraph,is_directed)
importFrom(igraph,is_simple)
importFrom(igraph,layout_nicely)
importFrom(igraph,simplify)
importFrom(igraph,vcount)
importFrom(igraph,vertex_attr)
importFrom(igraph,vertex_attr_names)
importFrom(igraph,which_mutual)
importFrom(lifecycle,deprecate_soft)
importFrom(lifecycle,deprecated)
importFrom(lifecycle,is_present)
importFrom(methods,setClass)
importFrom(methods,setOldClass)
importFrom(rlang,'%||%')
importFrom(rlang,exec)
importFrom(rlang,list2)
importFrom(rlang,warn)
importFrom(scales,alpha)
importFrom(scales,rescale)
importFrom(scales,squish)
importFrom(tidygraph,as.igraph)
importFrom(tidygraph,tbl_graph)
importFrom(utils,.DollarNames)
importFrom(utils,modifyList)
