% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gspace-misc.R
\docType{package}
\name{RGraphSpace-package}
\alias{RGraphSpace}
\alias{RGraphSpace-package}
\title{RGraphSpace: A Lightweight Interface Between 'igraph' and 'ggplot2' Graphics}
\description{
An interface to integrate 'igraph' and 'ggplot2' graphics within a normalized coordinate system. 'RGraphSpace' extends 'ggplot2' with graph-aware geometries optimized for large networks. The 'GraphSpace' class integrates directly with 'ggplot2' through specialized 'geoms' and lazy resolution of node attributes, supporting customization of aesthetics and visual styles. These 'geoms' use a dual-anchor normalization approach to align layers, particularly useful for analyses in which network elements must be spatially aligned with reference maps and images.
}
\section{Index}{

\tabular{ll}{
\link{GraphSpace}:
\tab Constructor of GraphSpace-class objects.\cr
\link{plotGraphSpace}:
\tab Wrapper function to plot GraphSpace objects in ggplot2.
}
Further information is available in the vignettes by typing
\code{vignette('RGraphSpace')}. Documented topics are also available in
HTML by typing \code{help.start()} and selecting the RGraphSpace package
from the menu.
}

\references{
Sysbiolab Team (2026). \emph{RGraphSpace: A lightweight interface between
'igraph' and 'ggplot2' graphics}. R package version 1.4.0 (Doi:
10.32614/CRAN.package.RGraphSpace),
\url{https://CRAN.R-project.org/package=RGraphSpace}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/sysbiolab/RGraphSpace}
  \item \url{https://sysbiolab.github.io/RGraphSpace/}
  \item Report bugs at \url{https://github.com/sysbiolab/RGraphSpace/issues}
}

}
\author{
\strong{Maintainer}: Mauro Castro \email{mauro.a.castro@gmail.com} (\href{https://orcid.org/0000-0003-4942-8131}{ORCID})

Authors:
\itemize{
  \item Sysbiolab Team
}

Other contributors:
\itemize{
  \item Flávio Kessler [contributor]
  \item Jonathan Back [contributor]
  \item Lana Querne [contributor]
  \item Victor Apolonio [contributor]
  \item Vinicius Chagas [contributor]
}

}
\keyword{internal}
