% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-nodespace.R
\name{GeomNodeSpace}
\alias{GeomNodeSpace}
\title{GeomNodeSpace: a ggplot2 prototype for GraphSpace-class methods}
\usage{
GeomNodeSpace
}
\description{
\code{GeomNodeSpace} is the underlying \link[ggplot2]{ggproto} object
used by \link{geom_nodespace} to draw node elements in a graph layout.

This geom is designed for network diagrams, where graph attributes
are often already in their final form (e.g., hex colors).
}
\section{Aesthetics}{


\code{GeomNodeSpace} understands ggplot2's conventions for point-like geoms.
}

\seealso{
\link{geom_nodespace}, \link[ggplot2]{geom_point}
}
