% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gspace-methods.R
\name{plot.GraphSpace}
\alias{plot.GraphSpace}
\title{Plot GraphSpace objects}
\usage{
\method{plot}{GraphSpace}(x, ...)
}
\arguments{
\item{x}{A \linkS4class{GraphSpace} class object.}

\item{...}{Additional arguments passed to the
\code{\link{plotGraphSpace}} function.}
}
\description{
Plot GraphSpace objects
}
\seealso{
\code{\link{plotGraphSpace}}
}
