% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gspace-classes.R
\docType{class}
\name{GraphSpace-class}
\alias{GraphSpace-class}
\title{GraphSpace: An S4 class for igraph objects}
\value{
An S4 class object.
}
\description{
GraphSpace: An S4 class for igraph objects
}
\section{Slots}{

\describe{
\item{\code{nodes}}{A data frame with xy-vertex coordinates.}

\item{\code{edges}}{A data frame with edges.}

\item{\code{graph}}{An igraph object.}

\item{\code{image}}{A raster background image matrix.}

\item{\code{pars}}{A list with parameters.}

\item{\code{misc}}{A list with intermediate objects for downstream methods.}
}}

\section{Constructor}{

see \code{\link{GraphSpace}} constructor.
}

