% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-edgespace.R
\docType{data}
\name{GeomEdgeSpace}
\alias{GeomEdgeSpace}
\title{GeomEdgeSpace: a ggplot2 prototype for GraphSpace-class methods}
\format{
An object of class \code{GeomEdgeSpace} (inherits from \code{GeomSegment}, \code{Geom}, \code{ggproto}, \code{gg}) of length 7.
}
\usage{
GeomEdgeSpace
}
\description{
\code{GeomEdgeSpace} is the underlying \link[ggplot2]{ggproto} object
used by \link{geom_edgespace} to draw edge elements in a graph layout.

This geom is designed for network diagrams, where graph attributes
are often already in their final form (e.g., hex colors).
}
\section{Aesthetics}{


\code{GeomEdgeSpace} understands ggplot2's conventions for segment-like geoms.
}

\seealso{
\link{geom_edgespace}, \link[ggplot2]{geom_segment}
}
\keyword{datasets}
