% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/geom-graphspace.R
\name{GeomGraphSpace}
\alias{GeomGraphSpace}
\title{GeomGraphSpace: a ggplot2 prototype for GraphSpace-class methods}
\usage{
GeomGraphSpace
}
\description{
\code{GeomGraphSpace} is the underlying \link[ggplot2]{ggproto} object
used by \link{geom_graphspace} to draw node and edge elements in a
graph layout.

This geom is designed for network diagrams, where graph attributes
are often already in their final form (e.g., hex colors).
}
\section{Aesthetics}{


\code{GeomGraphSpace} understands ggplot2's conventions for point-like geoms.
}

\seealso{
\link{geom_graphspace}, \link[ggplot2]{geom_point}
}
