% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gspace-misc.R
\docType{data}
\name{gtoys}
\alias{gtoys}
\alias{gtoy1}
\alias{gtoy2}
\title{Toy 'igraph' objects}
\format{
igraph
}
\source{
This package.
}
\usage{
data(gtoy1)
}
\value{
A pre-processed igraph object.
}
\description{
Small 'igraph' objects used for workflow demonstrations.
All graphs include 'x', 'y', and 'name' vertex attributes.
}
\examples{
library(RGraphSpace)
data(gtoy1)
data(gtoy2)
}
\keyword{gtoys}
