% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gspace-methods.R
\name{GraphSpace-internal}
\alias{GraphSpace-internal}
\alias{.DollarNames.GraphSpace}
\title{Internal methods for GraphSpace}
\usage{
\method{.DollarNames}{GraphSpace}(x, pattern = "")
}
\arguments{
\item{x, pattern}{Internal arguments.}
}
\description{
Exported solely to enable RStudio auto-completion
and should not be called directly by the user.
}
\keyword{internal}
